
require("./lib/jasmine-promise");
var Q = require("q");
var Queue = require("q/queue");
var Connection = require("../q-connection");

function makeChannel() {
    var sending = Queue();
    var receiving = Queue();
    return {
        l2r: {
            get: sending.get,
            put: receiving.put
        },
        r2l: {
            get: receiving.get,
            put: sending.put
        }
    };
}

function makePeers(local, remote) {
    var channel = makeChannel();
    return {
        local: Connection(channel.l2r, local),
        remote: Connection(channel.r2l, remote)
    }
}

describe("channel", function () {
    it("should send messages", function () {
        var channel = makeChannel();
        channel.l2r.put(10);
        channel.r2l.put(20);
        var a = channel.l2r.get().then(function (value) {
            expect(value).toBe(20);
        });
        var b = channel.r2l.get().then(function (value) {
            expect(value).toBe(10);
        });
        return Q.all([a, b]);
    })
});

describe("RPC", function () {
    var peers = makePeers({
        callback: function (callback) {
            return callback("Hello");
        }
    }, {
    });

    return peers.remote.send('callback', function (value) {
        console.log(value);
    });
});


